package com.flyer.foster.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.flyer.exception.BusinessException;
import com.flyer.foster.dto.RoleAddDTO;
import com.flyer.foster.dto.RoleMenuAddDTO;
import com.flyer.foster.dto.RoleRespDTO;
import com.flyer.foster.dto.RoleUpdateDTO;
import com.flyer.foster.entity.Role;
import com.flyer.foster.entity.RoleMenu;
import com.flyer.foster.enums.IsAdminEnum;
import com.flyer.foster.enums.StatusEnum;
import com.flyer.foster.enums.TFEnum;
import com.flyer.foster.mapper.IRoleMapper;
import com.flyer.foster.pojo.Dropdown;
import com.flyer.foster.service.IRoleMenuService;
import com.flyer.foster.service.IRoleService;
import com.flyer.foster.service.IUserRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 角色表 服务实现类
 * </p>
 *
 * @author flyer
 * @since 2024-05-06
 */
@Service
public class RoleServiceImpl extends ServiceImpl<IRoleMapper, Role> implements IRoleService {
    @Autowired
    private IRoleMenuService iRoleMenuService;

    @Autowired
    private IUserRoleService iUserRoleService;

    @Override
    public List<RoleRespDTO> getRoleList() {
        int tenantId = StpUtil.getTokenSession().getInt("tenantId");
        List<RoleRespDTO> list = new ArrayList<>();
        List<Role> roleList = this.lambdaQuery()
                .eq(Role::getTenantId, tenantId)
                .eq(Role::getIsAdmin, TFEnum.FALSE.getId())
                .list();
        RoleRespDTO roleRespDTO;
        for (Role role : roleList) {
            roleRespDTO = new RoleRespDTO();
            BeanUtil.copyProperties(role, roleRespDTO);
            list.add(roleRespDTO);
        }
        return list;
    }

    @Transactional
    @Override
    public boolean addRole(RoleAddDTO addDTO) {
        int tenantId = StpUtil.getTokenSession().getInt("tenantId");
        addDTO.setRoleName(addDTO.getRoleName().trim());
        addDTO.setRoleCode(addDTO.getRoleCode().trim());
        if (addDTO.getMenuIdList().size() == 0) {
            throw new BusinessException("请选择菜单");
        }
        // 1.判断（角色名称+角色编码）是否存在
        Role role = this.lambdaQuery()
                .eq(Role::getRoleName, addDTO.getRoleName())
                .eq(Role::getRoleCode, addDTO.getRoleCode())
                .one();
        if (role != null) {
            throw new BusinessException("角色编码+角色名称不能重复");
        }
        role = new Role();
        BeanUtil.copyProperties(addDTO, role);
        role.setTenantId(tenantId);
        // 新增角色
        this.save(role);
        // 保存角色菜单
        RoleMenu roleMenu;
        for (Integer menuId : addDTO.getMenuIdList()) {
            roleMenu = new RoleMenu();
            roleMenu.setRoleId(role.getId());
            roleMenu.setMenuId(menuId);
            iRoleMenuService.save(roleMenu);
        }
        return true;
    }

    @Transactional
    @Override
    public boolean updateRole(RoleUpdateDTO updateDTO) {
        int tenantId = StpUtil.getTokenSession().getInt("tenantId");
        Role role = this.lambdaQuery()
                .eq(Role::getId, updateDTO.getRoleId())
                .eq(Role::getTenantId, tenantId)
                .one();
        if (role == null) {
            throw new BusinessException(StrUtil.format("无效的角色id:{}", updateDTO.getRoleId()));
        }

        // 删除菜单后新增
        RoleMenuAddDTO roleMenuAddDTO = new RoleMenuAddDTO();
        roleMenuAddDTO.setRoleId(updateDTO.getRoleId());
        roleMenuAddDTO.setMenuIdList(updateDTO.getMenuIdList());
        iRoleMenuService.addRoleMenu(roleMenuAddDTO);

        // 没有修改则不作修改
        if (role.getRoleName().equals(updateDTO.getRoleName()) && role.getRoleCode().equals(updateDTO.getRoleCode())) {
            return true;
        } else if (StrUtil.isBlank(updateDTO.getRoleName()) && StrUtil.isBlank(updateDTO.getRoleCode())) {
            return true;
        } else {
            Role one = this.lambdaQuery()
                    .eq(Role::getRoleName, updateDTO.getRoleName())
                    .eq(Role::getRoleCode, updateDTO.getRoleCode())
                    .eq(Role::getTenantId, tenantId)
                    .ne(Role::getId, updateDTO.getRoleId())
                    .one();
            if (one != null) {
                throw new BusinessException(StrUtil.format("角色名称：{}+角色编码：{}已存在", role.getRoleName(), role.getRoleCode()));
            }
            role.setRoleName(updateDTO.getRoleName());
            role.setRoleCode(updateDTO.getRoleCode());
            return this.updateById(role);
        }
    }

    @Transactional
    @Override
    public boolean deleteRole(List<Integer> roleIdList) {
        int tenantId = StpUtil.getTokenSession().getInt("tenantId");
        // 物理删除角色
        baseMapper.deleteByIdList(roleIdList);
        // 物理删除角色菜单
        iRoleMenuService.deleteByRoleIdList(roleIdList);
        // 物理删除用户角色
        iUserRoleService.deleteByRoleIdList(roleIdList);
        return true;
    }

    @Override
    public List<Dropdown> getDropdownList() {
        int tenantId = StpUtil.getTokenSession().getInt("tenantId");
        List<Role> roleList = this.lambdaQuery()
                .eq(Role::getTenantId, tenantId)
                .eq(Role::getIsAdmin, IsAdminEnum.NO.getId())
                .eq(Role::getStatus, StatusEnum.ENABLE.getId())
                .list();
        List<Dropdown> dropdownList = new ArrayList<>();
        Dropdown dropdown;
        for (Role role : roleList) {
            dropdown = new Dropdown();
            Map<String, String> mapping = new HashMap<>();
            mapping.put("roleName", "name");
            BeanUtil.copyProperties(role, dropdown, CopyOptions.create().setFieldMapping(mapping));
            dropdownList.add(dropdown);
        }
        return dropdownList;
    }
}
